// exception.cc 
//	Entry point into the Nachos kernel from user programs.
//	There are two kinds of things that can cause control to
//	transfer back to here from user code:
//
//	syscall -- The user code explicitly requests to call a procedure
//	in the Nachos kernel.  Right now, the only function we support is
//	"Halt".
//
//	exceptions -- The user code does something that the CPU can't handle.
//	For instance, accessing memory that doesn't exist, arithmetic errors,
//	etc.  
//
//	Interrupts (which can also cause control to transfer from user
//	code into the Nachos kernel) are handled elsewhere.
//
// For now, this only handles the Halt() system call.
// Everything else core dumps.
//
// Copyright (c) 1992-1993 The Regents of the University of California.
// All rights reserved.  See copyright.h for copyright notice and limitation 
// of liability and disclaimer of warranty provisions.

#include "copyright.h"
#include "system.h"
#include "syscall.h"

//EECE 315 assignment 3
#include "bitmap.h"
extern BitMap* bitmap_page_in_use;
extern BitMap* bitmap_process;
void processCreator(int arg)
{
	currentThread->space->InitRegisters();
	currentThread->space->RestoreState();
	printf("Thread started: %s\n", currentThread->getName());
	machine->Run();
	ASSERT(FALSE);
}
//----------------------------------------------------------------------
// ExceptionHandler
// 	Entry point into the Nachos kernel.  Called when a user program
//	is executing, and either does a syscall, or generates an addressing
//	or arithmetic exception.
//
// 	For system calls, the following is the calling convention:
//
// 	system call code -- r2
//		arg1 -- r4
//		arg2 -- r5
//		arg3 -- r6
//		arg4 -- r7
//
//	The result of the system call, if any, must be put back into r2. 
//
// And don't forget to increment the pc before returning. (Or else you'll
// loop making the same system call forever!
//
//	"which" is the kind of exception.  The list of possible exceptions 
//	are in machine.h.
//----------------------------------------------------------------------

void
ExceptionHandler(ExceptionType which)
{
    int type = machine->ReadRegister(2);

    if ((which == SyscallException) && (type == SC_Halt)) {
	DEBUG('a', "Shutdown, initiated by user program.\n");
   	interrupt->Halt();
    }
	else if((which == SyscallException) && (type == SC_Exit))
	{
		IntStatus oldLevel = interrupt->SetLevel(IntOff);
		//EECE 315 assignment 3, for SC_Exit task 2
		int stat = machine->ReadRegister(4);
		machine->WriteRegister(2, stat);
		printf("============================\n");
		printf("exit status: %d\n", stat);
		printf("Thread exit: %s\n", currentThread->getName());
		printf("Spare pages: %d\n", bitmap_page_in_use->NumClear());
		printf("Spare processes: %d\n", bitmap_process->NumClear());
		printf("============================\n");
		ASSERT(bitmap_process->Test(currentThread->SpaceId));
		bitmap_process->Clear(currentThread->SpaceId);
		currentThread->Finish();
		(void) interrupt->SetLevel(oldLevel);
	}
	else if((which == SyscallException) && (type == SC_Exec))
	{
		IntStatus oldLevel = interrupt->SetLevel(IntOff);
		//EECE 315 assignment 3, for SC_Exec task 3
		int name_addr = machine->ReadRegister(4);
		char* name = new char[100];
		for(int i = 0; i < 100; i++)
		{
			int ch = 0;
			machine->ReadMem(name_addr + i, 1, &ch);
			name[i] = (char) ch;
			if(name[i] == '\0')	break;
		}
		OpenFile *executable = fileSystem->Open(name);
    		AddrSpace *space;

    		if (executable == NULL) 
		{
			printf("Unable to open file %s\n", name);
			machine->WriteRegister(2, 0);
    		}
		space = new AddrSpace(executable);
		Thread* newThread = new Thread(name);
		newThread->space = space;

		delete executable;
		int SpaceId = bitmap_process->Find();
		printf("SpaceId: %d for Thread: %s\n", SpaceId, newThread->getName());
		newThread->SpaceId = SpaceId;
		newThread->Fork(processCreator, 0);

		machine->WriteRegister(2, SpaceId);
		int pc = machine->ReadRegister(PCReg);
		machine->WriteRegister(PrevPCReg, pc);
		pc = machine->ReadRegister(NextPCReg);
		machine->WriteRegister(PCReg, pc);
		pc += 4;
		machine->WriteRegister(NextPCReg, pc);
		(void) interrupt->SetLevel(oldLevel);
	}
	else if(which == BusErrorException)
	{
		printf("Out of memory! Spare pages: %d\n", bitmap_page_in_use->NumClear());
		ASSERT(FALSE);
	}
	else if((which == PageFaultException))
    	{
    		
    		
    		IntStatus oldLevel = interrupt->SetLevel(IntOff);
    		printf("page Fault execption triggered\n");
    		
    		//TODO: EECE 315 Assignment 4
    		//use preparePage function to allocate physical memory on page fault
    		//Refer machine.h to see which register to read for bad address
    		
    		
    		(void) interrupt->SetLevel(oldLevel);
    	}
	else 
	{
		printf("Unexpected user mode exception %d %d\n", which, type);
		ASSERT(FALSE);
    	}
}
